import resolveInclusions from "../utils/resolveInclusions";
import { create } from "xmlbuilder2";
import SaxonJS from "saxon-js";
import crypto from "crypto";
import { DOMParser, XMLSerializer } from "@xmldom/xmldom";
class FacturaCfdi {
    constructor(config_cfdi) {
        this.node_comprobante = {
            serie: "",
            folio: "",
            fecha: "",
            subtotal: "",
            formaPago: "",
            total: "",
            metodoPago: "PUE",
            lugarExpedicion: "",
        };
        this.node_informacion_global = {
            exist: false,
            periodicidad: "",
            meses: "",
            anio: "",
        };
        this.node_relacionados = [];
        this.node_emisor = {
            rfc: "",
            nombre: "",
            regimenFiscal: "",
        };
        this.node_receptor = {
            rfc: "",
            nombre: "",
            usoCfdi: "",
            domicilioFiscal: "",
            regimenFiscal: "",
        };
        this.node_conceptos = [];
        this.config_cfdi = config_cfdi;
    }
    createNodeComprobante(options) {
        this.node_comprobante = options;
    }
    createNodeInformacionGlobal(options) {
        this.node_informacion_global = options;
        this.node_informacion_global.exist = true;
    }
    createNodeRelacionados(options) {
        this.node_relacionados.push(options);
    }
    createNodeEmisor(options) {
        this.node_emisor = options;
    }
    createNodeReceptor(options) {
        this.node_receptor = options;
    }
    createNodeConcepto(options) {
        this.node_conceptos.push(options);
    }
    createXml() {
        return this.generateXml();
    }
    async createXmlSellado() {
        const xml = this.generateXml();
        return this.generateCadenaOriginal(xml)
            .then((sign) => {
            const parser = new DOMParser();
            const xmlDoc = parser.parseFromString(xml, "application/xml");
            const comprobanteElement = xmlDoc.getElementsByTagName("cfdi:Comprobante")[0];
            if (comprobanteElement) {
                comprobanteElement.setAttribute("Sello", sign);
            }
            const serializer = new XMLSerializer();
            return serializer.serializeToString(xmlDoc);
        })
            .catch((err) => {
            throw new Error(err.message);
        });
    }
    generateXml() {
        const doc = create({
            version: "1.0",
        }).ele("cfdi:Comprobante", this.generateNodeComprobante());
        if (this.node_informacion_global.exist) {
            doc.ele("cfdi:InformacionGlobal", {
                Periodicidad: this.node_informacion_global.periodicidad,
                Meses: this.node_informacion_global.meses,
                Año: this.node_informacion_global.anio,
            });
        }
        if (this.node_relacionados.length > 0) {
            for (const relation of this.node_relacionados) {
                const ele_relacionados = doc.ele("cfdi:CfdiRelacionados", {
                    TipoRelacion: relation.tipoRelacion,
                });
                for (const uuid of relation.uuids) {
                    ele_relacionados.ele("cfdi:CfdiRelacionado", {
                        UUID: uuid,
                    });
                }
            }
        }
        doc.ele("cfdi:Emisor", this.generateNodeEmisor());
        doc.ele("cfdi:Receptor", this.generateNodeReceptor());
        const ele_conceptos = doc.ele("cfdi:Conceptos");
        for (const concepto of this.node_conceptos) {
            const ele_concepto = ele_conceptos.ele("cfdi:Concepto", this.generateNodeConcepto(concepto.concepto));
            if ("impuestos" in concepto && concepto.concepto.objetoImp !== "01") {
                const ele_imp = ele_concepto.ele("cfdi:Impuestos");
                if ("traslados" in concepto.impuestos && concepto.impuestos.traslados.length > 0) {
                    const ele_tras = ele_imp.ele("cfdi:Traslados");
                    for (const traslado of concepto.impuestos.traslados) {
                        ele_tras.ele("cfdi:Traslado", this.generateNodeTraslado(traslado));
                    }
                }
                if ("retenciones" in concepto.impuestos && concepto.impuestos.retenciones.length > 0) {
                    const ele_ret = ele_imp.ele("cfdi:Retenciones");
                    for (const retencion of concepto.impuestos.retenciones) {
                        ele_ret.ele("cfdi:Retencion", this.generateNodeRetencion(retencion));
                    }
                }
            }
            if ("aCuentaTerceros" in concepto) {
                ele_concepto.ele("cfdi:ACuentaTerceros", this.generateNodeACuentaTerceros(concepto.aCuentaTerceros));
            }
            if ("informacionAduanera" in concepto) {
                ele_concepto.ele("cfdi:InformacionAduanera", {
                    NumeroPedimento: concepto.informacionAduanera.numeroPedimento,
                });
            }
            if ("cuentaPredial" in concepto) {
                ele_concepto.ele("cfdi:CuentaPredial", {
                    Numero: concepto.cuentaPredial.numero,
                });
            }
            if ("complementoConcepto" in concepto && concepto.complementoConcepto.length > 0) {
                const ele_compl_conc = ele_concepto.ele("cfdi:ComplementoConcepto");
                for (const comple of concepto.complementoConcepto) {
                    ele_compl_conc.ele(comple.nodo, comple.atributos);
                }
            }
            if ("parte" in concepto && concepto.parte.length > 0) {
                for (const part of concepto.parte) {
                    const ele_parte = ele_concepto.ele("cfdi:Parte", this.generateNodeParte(part.concepto));
                    if ("informacionAduanera" in part) {
                        ele_parte.ele("cfdi:InformacionAduanera", {
                            NumeroPedimento: part.informacionAduanera.numeroPedimento,
                        });
                    }
                }
            }
        }
        const createNodeImpuestos = this.createNodeImpuestos();
        if (createNodeImpuestos.total_impuestos_retenidos.total > 0 || createNodeImpuestos.total_impuestos_trasladados.total > 0) {
            const ele_impuestos = doc.ele("cfdi:Impuestos", {
                TotalImpuestosRetenidos: createNodeImpuestos.total_impuestos_retenidos.total.toFixed(2),
                TotalImpuestosTrasladados: createNodeImpuestos.total_impuestos_trasladados.total.toFixed(2),
            });
            if (createNodeImpuestos.total_impuestos_retenidos.data.length > 0) {
                const ele_ret = ele_impuestos.ele("cfdi:Retenciones");
                for (const ret of createNodeImpuestos.total_impuestos_retenidos.data) {
                    ele_ret.ele("cfdi:Retencion", {
                        Importe: ret.importe,
                        Impuesto: ret.impuesto,
                    });
                }
            }
            if (createNodeImpuestos.total_impuestos_trasladados.data.length > 0) {
                const ele_tras = ele_impuestos.ele("cfdi:Traslados");
                for (const tras of createNodeImpuestos.total_impuestos_trasladados.data) {
                    ele_tras.ele("cfdi:Traslado", {
                        Base: tras.base,
                        Impuesto: tras.impuesto,
                        TipoFactor: tras.tipoFactor,
                        TasaOCuota: parseFloat(tras.tasaOCuota).toFixed(6),
                        Importe: tras.importe,
                    });
                }
            }
        }
        return doc.end({ prettyPrint: true });
    }
    generateNodeComprobante() {
        const node_comprobante = {
            "xsi:schemaLocation": "http://www.sat.gob.mx/cfd/4 http://www.sat.gob.mx/sitio_internet/cfd/4/cfdv40.xsd",
            "xmlns:cfdi": "http://www.sat.gob.mx/cfd/4",
            "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
            Version: "4.0",
            Serie: this.node_comprobante.serie,
            Folio: this.node_comprobante.folio,
            Fecha: this.node_comprobante.fecha,
            SubTotal: parseFloat(this.node_comprobante.subtotal).toFixed(2),
            Moneda: this.node_comprobante.moneda ?? "MXN",
            FormaPago: this.node_comprobante.formaPago,
            Total: parseFloat(this.node_comprobante.total).toFixed(2),
            MetodoPago: this.node_comprobante.metodoPago,
            TipoDeComprobante: this.node_comprobante.tipoDeComprobante ?? "I",
            LugarExpedicion: this.node_comprobante.lugarExpedicion,
            NoCertificado: this.config_cfdi.getCert().noCertificado,
            Certificado: this.config_cfdi
                .getCert()
                .pem.replace("-----BEGIN CERTIFICATE-----", "")
                .replace("-----END CERTIFICATE-----", "")
                .replace(/(\r\n|\n|\r)/gm, ""),
            Exportacion: this.node_comprobante.exportacion ?? "01",
        };
        if ("tipoCambio" in this.node_comprobante && node_comprobante.Moneda !== "MXN") {
            node_comprobante.TipoCambio = this.node_comprobante.tipoCambio;
        }
        if ("condicionesDePago" in this.node_comprobante) {
            node_comprobante.CondicionesDePago = this.node_comprobante.condicionesDePago;
        }
        if ("descuento" in this.node_comprobante && parseFloat(this.node_comprobante.descuento) > 0) {
            node_comprobante.Descuento = this.node_comprobante.descuento;
        }
        return node_comprobante;
    }
    generateNodeEmisor() {
        const node_attr = {
            Rfc: this.node_emisor.rfc,
            Nombre: this.node_emisor.nombre,
            RegimenFiscal: this.node_emisor.regimenFiscal,
        };
        if ("facAtrAdquirente" in this.node_emisor) {
            node_attr.FacAtrAdquirente = this.node_emisor.facAtrAdquirente.padStart(10, "0");
        }
        return node_attr;
    }
    generateNodeReceptor() {
        const node_attr = {
            Rfc: this.node_receptor.rfc,
            Nombre: this.node_receptor.nombre,
            UsoCFDI: this.node_receptor.usoCfdi,
            DomicilioFiscalReceptor: this.node_receptor.domicilioFiscal,
            RegimenFiscalReceptor: this.node_receptor.regimenFiscal,
        };
        if ("residenciaFiscal" in this.node_receptor) {
            node_attr.ResidenciaFiscal = this.node_receptor.residenciaFiscal;
        }
        if ("numRegIdTrib" in this.node_receptor) {
            node_attr.NumRegIdTrib = this.node_receptor.numRegIdTrib;
        }
        return node_attr;
    }
    generateNodeConcepto(concepto) {
        const node_attr = {
            ClaveProdServ: concepto.claveProdServ,
            Cantidad: concepto.cantidad,
            ClaveUnidad: concepto.claveUnidad,
            Descripcion: concepto.descripcion,
            Importe: parseFloat(concepto.importe).toFixed(2),
            ObjetoImp: concepto.objetoImp ?? "02",
            ValorUnitario: parseFloat(concepto.valorUnitario).toFixed(2),
            Unidad: concepto.unidad,
        };
        if ("noIdentificacion" in concepto) {
            node_attr.NoIdentificacion = concepto.noIdentificacion;
        }
        if ("descuento" in concepto && parseFloat(concepto.descuento) > 0) {
            node_attr.Descuento = parseFloat(concepto.descuento).toFixed(2);
        }
        return node_attr;
    }
    generateNodeTraslado(traslado) {
        const node_attr = {
            Base: parseFloat(traslado.base).toFixed(2),
            Impuesto: traslado.impuesto,
            TipoFactor: traslado.tipoFactor,
            Importe: parseFloat(traslado.importe).toFixed(2),
        };
        if ("tasaOCuota" in traslado) {
            node_attr.TasaOCuota = parseFloat(traslado.tasaOCuota).toFixed(6);
        }
        return node_attr;
    }
    generateNodeRetencion(retencion) {
        return {
            Base: parseFloat(retencion.base).toFixed(2),
            Impuesto: retencion.impuesto,
            TipoFactor: retencion.tipoFactor,
            TasaOCuota: parseFloat(retencion.tasaOCuota).toFixed(6),
            Importe: parseFloat(retencion.importe).toFixed(2),
        };
    }
    generateNodeACuentaTerceros(aCuentaTerceros) {
        return {
            RfcACuentaTerceros: aCuentaTerceros.rfcACuentaTerceros,
            NombreACuentaTerceros: aCuentaTerceros.nombreACuentaTerceros,
            DomicilioFiscalACuentaTerceros: aCuentaTerceros.domicilioFiscalACuentaTerceros,
            RegimenFiscalACuentaTerceros: aCuentaTerceros.regimenFiscalACuentaTerceros,
        };
    }
    generateNodeParte(parte) {
        const node_attr = {
            Cantidad: parte.cantidad,
            ClaveProdServ: parte.claveProdServ,
            Descripcion: parte.descripcion,
        };
        if ("unidad" in parte) {
            node_attr.Unidad = parte.unidad;
        }
        if ("noIdentificacion" in parte) {
            node_attr.NoIdentificacion = parte.noIdentificacion;
        }
        if ("valorUnitario" in parte && parseFloat(parte.valorUnitario) > 0) {
            node_attr.ValorUnitario = parseFloat(parte.valorUnitario).toFixed(2);
        }
        if ("importe" in parte && parseFloat(parte.importe) > 0) {
            node_attr.Importe = parseFloat(parte.importe).toFixed(2);
        }
        return node_attr;
    }
    createNodeImpuestos() {
        const imp_obj = {
            total_impuestos_retenidos: {
                total: 0,
                data: [],
            },
            total_impuestos_trasladados: {
                total: 0,
                data: [],
            },
        };
        for (const concepto of this.node_conceptos) {
            if ("impuestos" in concepto) {
                if ("traslados" in concepto.impuestos && concepto.impuestos.traslados.length > 0) {
                    imp_obj.total_impuestos_trasladados.data.push(...concepto.impuestos.traslados.map((traslado) => ({
                        base: parseFloat(traslado.base).toFixed(2),
                        importe: parseFloat(traslado.importe).toFixed(2),
                        impuesto: traslado.impuesto,
                        tasaOCuota: parseFloat(traslado.tasaOCuota).toFixed(6),
                        tipoFactor: traslado.tipoFactor,
                    })));
                    imp_obj.total_impuestos_trasladados.total += concepto.impuestos.traslados.reduce((acc, cur) => acc + parseFloat(cur.importe ?? "0"), 0);
                }
                if ("retenciones" in concepto.impuestos && concepto.impuestos.retenciones.length > 0) {
                    imp_obj.total_impuestos_retenidos.data.push(...concepto.impuestos.retenciones.map((retencion) => ({
                        importe: parseFloat(retencion.importe).toFixed(2),
                        impuesto: retencion.impuesto,
                    })));
                    imp_obj.total_impuestos_retenidos.total += concepto.impuestos.retenciones.reduce((acc, cur) => acc + parseFloat(cur.importe ?? "0"), 0);
                }
            }
        }
        return imp_obj;
    }
    async generateCadenaOriginal(xml) {
        try {
            const cadenaOriginalXslt = await resolveInclusions();
            let result = SaxonJS.XPath.evaluate(`transform(
        map {
          'source-node' : parse-xml($xml),
          'stylesheet-text' : $xslt,
          'delivery-format' : 'serialized'
          }
      )?output`, [], {
                params: {
                    xml: xml,
                    xslt: cadenaOriginalXslt,
                },
            });
            const sign = crypto.createSign("SHA256");
            sign.update(result);
            sign.end();
            const signature = sign.sign(this.config_cfdi.getKeyPem(), "base64");
            return signature;
        }
        catch (error) {
            throw new Error(`Error generating Cadena Original: ${error}`);
        }
    }
}
export default FacturaCfdi;
